/*
** 1998-09-23 -	No program is complete without an "About" command.
** 1999-03-13 -	Now uses a multi dialog, which allows setting a sensible title. :)
** 1999-03-21 -	Brought the copyright into 1999, and added fun page flipping. :)
** 1999-06-20 -	Adapted for the new dialog module, and made asynchronous for fun.
** 2002-06-07 -	Redid it with a notebook. Cleaner, and required anyway. Heh.
*/

#include "gentoo.h"
#include "dialog.h"
#include "gfam.h"

#include "cmd_about.h"

/* ----------------------------------------------------------------------------------------- */

#include "graphics/icon_gentoo.xpm"

/* ----------------------------------------------------------------------------------------- */

struct body {
	GtkWidget	*notebook;
	GtkWidget	*page[2];
	guint		index;
	guint		handle;
};

/* ----------------------------------------------------------------------------------------- */

static gint timeout_trigger(gpointer user)
{
	struct body	*body = user;

	gtk_notebook_set_current_page(GTK_NOTEBOOK(body->notebook), body->index ^= 1);
	return TRUE;
}

/* 1999-06-19 -	This gets called as the dialog finally closes. */
static void timeout_remove(gint button, gpointer user)
{
	struct body	*body = user;

	g_source_remove(GPOINTER_TO_UINT(body->handle));
	g_free(body);
}

/* 2008-03-16 -	Returns a label describing the current CHARSET value (I think). */
static GtkWidget * get_charset_label(void)
{
	const gchar	*charset = NULL;
	gchar		buf[1024];

	g_get_charset(&charset);
	g_snprintf(buf, sizeof buf, _("Native CHARSET: \"%s\"."), charset);
	return gtk_label_new(buf);
}

/* 2008-08-09 -	Returns a label describing the current filename encoding. */
static GtkWidget * get_filename_label(void)
{
	const gchar	**enc = NULL;

	g_get_filename_charsets(&enc);
	if(enc != NULL && enc[0] != NULL)
	{
		gchar	buf[1024];

		g_snprintf(buf, sizeof buf, _("Filename encoding: \"%s\"."), enc[0]);
		return gtk_label_new(buf);
	}
	return NULL;
}

gint cmd_about(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
{
	gchar		buf[64], page2[768];
	GtkWidget	*w;
	GdkPixmap	*pmap;
	GdkBitmap	*mask;
	struct body	*body;

	body = g_malloc(sizeof *body);
	body->index = 0U;

	g_snprintf(buf, sizeof buf, _("Version %s (GTK+ version %d.%d.%d)."), VERSION, gtk_major_version, gtk_minor_version, gtk_micro_version);

	body->notebook = gtk_notebook_new();
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(body->notebook), FALSE);
	body->page[0] = gtk_vbox_new(FALSE, 0);
	if((pmap = gdk_pixmap_create_from_xpm_d(GTK_WIDGET(min->gui->window)->window, &mask, NULL, icon_gentoo_xpm)) != NULL)
	{
		w = gtk_image_new_from_pixmap(pmap, mask);
		gtk_box_pack_start(GTK_BOX(body->page[0]), w, FALSE, FALSE, 5);
	}
	w = gtk_label_new(buf);
	gtk_box_pack_start(GTK_BOX(body->page[0]), w, FALSE, FALSE, 0);
	w = gtk_label_new(_("(c) 1998-2011 by Emil Brink, Obsession Development.\n"));
	gtk_box_pack_start(GTK_BOX(body->page[0]), w, FALSE, FALSE, 0);
	w = gtk_label_new(_("This is free software, and there is ABSOLUTELY NO\n"
			"WARRANTY. Read the file COPYING for more details.\n"));
	gtk_box_pack_start(GTK_BOX(body->page[0]), w, FALSE, FALSE, 0);

	/* Inform user which interface strings are being used. */
#if defined ENABLE_NLS
	w = gtk_label_new(_("NLS: Supported, using built-in English strings."));
#else
	w = gtk_label_new("NLS: Not supported, using built-in English strings.");
#endif
	gtk_box_pack_start(GTK_BOX(body->page[0]), w, FALSE, FALSE, 0);

	w = get_charset_label();
	gtk_box_pack_start(GTK_BOX(body->page[0]), w, FALSE, FALSE, 0);
	w = get_filename_label();
	if(w != NULL)
		gtk_box_pack_start(GTK_BOX(body->page[0]), w, FALSE, FALSE, 0);

	gtk_notebook_append_page(GTK_NOTEBOOK(body->notebook), body->page[0], NULL);
	g_snprintf(page2, sizeof page2, _("The author of gentoo can be reached via Internet\n"
					"e-mail at <a href=\"mailto:%s\">%s</a>; feel free to let\n"
					"me know what you think of this software, give\n"
					"suggestions/bug reports, and so on.\n\n"

					"The latest release of gentoo can always be down-\n"
					"loaded from the official gentoo project homepage at\n"
					"<a href=\"%s\">%s</a>. New releases are\n"
					"typically announced on the Freshmeat service at\n"
					"<a href=\"http://freshmeat.net/\">freshmeat.net</a>."),
					PACKAGE_BUGREPORT, PACKAGE_BUGREPORT,
					PACKAGE_URL, PACKAGE_URL);
	body->page[1] = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(body->page[1]), page2);
	gtk_notebook_append_page(GTK_NOTEBOOK(body->notebook), body->page[1], NULL);

	body->handle = g_timeout_add(10000, timeout_trigger, body);
	dlg_dialog_async_new(body->notebook, _("About gentoo"), _("_OK (Wait for More)"), timeout_remove, body);

	return 1;
}
