/*
** 1998-05-25 -	The PARENT native command, which simply causes the currently active
**		dir pane to go up one level, to its parent directory.
** 1999-07-03 -	Rewrote entire function, should now work a little bit better. I hope.
*/

#include "gentoo.h"

#include <stdlib.h>

#include "cmdseq.h"
#include "strutil.h"

#include "cmd_parent.h"

/* ----------------------------------------------------------------------------------------- */

/* 1999-07-03 -	Go up one level in the source pane. Rewritten to add a little more armor
**		plating. Still far from bullet-proof, though...
** 2010-06-05 -	Rewritten to be lot more GIO, no more string-level parent computation.
*/
gint cmd_parent(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
{
	GFile	*parent;
	gint	ret = 0;

	if(src->dir.root == NULL)
		return 0;

	parent = g_file_get_parent(src->dir.root);
	if(parent != NULL)
	{
		gchar	*uri;

		if((uri = g_file_get_uri(parent)) != NULL)
		{
			ret = csq_execute_format(min, "DirEnter 'dir=%s'", stu_escape(uri));
			g_free(uri);
		}
	}
	return ret;
}
